@@ -1,6 +1,8 @@
 package jwe_test
 
 import (
+	"bytes"
+	"context"
 	"crypto"
 	"crypto/ecdsa"
 	"crypto/elliptic"
@@ -8,18 +10,19 @@ import (
 	"crypto/rsa"
 	"encoding/base64"
 	"fmt"
+	"math"
 	"os"
 	"strings"
 	"testing"
 	"time"
 
-	"github.com/lestrrat-go/jwx/internal/json"
-	"github.com/lestrrat-go/jwx/internal/jwxtest"
+	"github.com/lestrrat-go/jwx/v2/internal/json"
+	"github.com/lestrrat-go/jwx/v2/internal/jwxtest"
 
-	"github.com/lestrrat-go/jwx/jwa"
-	"github.com/lestrrat-go/jwx/jwe"
-	"github.com/lestrrat-go/jwx/jwk"
-	"github.com/lestrrat-go/jwx/x25519"
+	"github.com/lestrrat-go/jwx/v2/jwa"
+	"github.com/lestrrat-go/jwx/v2/jwe"
+	"github.com/lestrrat-go/jwx/v2/jwk"
+	"github.com/lestrrat-go/jwx/v2/x25519"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -43,8 +46,8 @@ func init() {
       "qi":"VIMpMYbPf47dT1w_zDUXfPimsSegnMOA1zTaX7aGk_8urY6R8-ZW1FxU7AlWAyLWybqq6t16VFd7hQd0y6flUK4SlOydB61gwanOsXGOAOv82cHq0E3eL4HrtZkUuKvnPrMnsUUFlfUdybVzxyjz9JF_XyaY14ardLSjf4L_FNY"
      }`)
 
-	privkey := jwk.NewRSAPrivateKey()
-	if err := json.Unmarshal(jwkstr, privkey); err != nil {
+	privkey, err := jwk.ParseKey(jwkstr)
+	if err != nil {
 		panic(err)
 	}
 
@@ -127,7 +130,7 @@ func TestParse(t *testing.T) {
 			return
 		}
 
-		buf, err := jwe.JSON(msg)
+		buf, err := json.Marshal(msg)
 		if !assert.NoError(t, err, "Serializing to JSON format should succeed") {
 			return
 		}
@@ -160,8 +163,8 @@ func TestParse_RSAES_OAEP_AES_GCM(t *testing.T) {
       "qi":"VIMpMYbPf47dT1w_zDUXfPimsSegnMOA1zTaX7aGk_8urY6R8-ZW1FxU7AlWAyLWybqq6t16VFd7hQd0y6flUK4SlOydB61gwanOsXGOAOv82cHq0E3eL4HrtZkUuKvnPrMnsUUFlfUdybVzxyjz9JF_XyaY14ardLSjf4L_FNY"
      }`)
 
-	privkey := jwk.NewRSAPrivateKey()
-	if !assert.NoError(t, json.Unmarshal(jwkstr, privkey), `parsing jwk should succeed`) {
+	privkey, err := jwk.ParseKey(jwkstr)
+	if !assert.NoError(t, err, `parsing jwk should succeed`) {
 		return
 	}
 
@@ -170,47 +173,38 @@ func TestParse_RSAES_OAEP_AES_GCM(t *testing.T) {
 		return
 	}
 
-	msg, err := jwe.ParseString(serialized)
-	if !assert.NoError(t, err, "parse successful") {
+	msg := jwe.NewMessage()
+	plaintext, err := jwe.Decrypt([]byte(serialized), jwe.WithKey(jwa.RSA_OAEP, rawkey), jwe.WithMessage(msg))
+	if !assert.NoError(t, err, "jwe.Decrypt should be successful") {
 		return
 	}
 
-	{
-		buf, _ := json.MarshalIndent(msg, "", "  ")
-		t.Logf("%s", buf)
-	}
-
 	if !assert.Equal(t, 1, len(msg.Recipients()), "message recipients header length is 1") {
 		return
 	}
 
-	plaintext, err := msg.Decrypt(jwa.RSA_OAEP, rawkey)
-	if !assert.NoError(t, err, "Decrypt message succeeded") {
-		return
-	}
-
 	if !assert.Equal(t, payload, string(plaintext), "decrypted value does not match") {
 		return
 	}
 
-	serializers := []struct {
+	templates := []*struct {
 		Name     string
-		Func     func(*jwe.Message) ([]byte, error)
+		Options  []jwe.EncryptOption
 		Expected string
 	}{
 		{
 			Name:     "Compact",
-			Func:     func(m *jwe.Message) ([]byte, error) { return jwe.Compact(m) },
+			Options:  []jwe.EncryptOption{jwe.WithCompact()},
 			Expected: serialized,
 		},
 		{
 			Name:     "JSON",
-			Func:     func(m *jwe.Message) ([]byte, error) { return jwe.JSON(m) },
+			Options:  []jwe.EncryptOption{jwe.WithJSON()},
 			Expected: `{"ciphertext":"5eym8TW_c8SuK0ltJ3rpYIzOeDQz7TALvtu6UG9oMo4vpzs9tX_EFShS8iB7j6jiSdiwkIr3ajwQzaBtQD_A","iv":"48V1_ALb6US04U3b","protected":"eyJhbGciOiJSU0EtT0FFUCIsImVuYyI6IkEyNTZHQ00ifQ","header":{"alg":"RSA-OAEP"},"encrypted_key":"OKOawDo13gRp2ojaHV7LFpZcgV7T6DVZKTyKOMTYUmKoTCVJRgckCL9kiMT03JGeipsEdY3mx_etLbbWSrFr05kLzcSr4qKAq7YN7e9jwQRb23nfa6c9d-StnImGyFDbSv04uVuxIp5Zms1gNxKKK2Da14B8S4rzVRltdYwam_lDp5XnZAYpQdb76FdIKLaVmqgfwX7XWRxv2322i-vDxRfqNzo_tETKzpVLzfiwQyeyPGLBIO56YJ7eObdv0je81860ppamavo35UgoRdbYaBcoh9QcfylQr66oc6vFWXRcZ_ZT2LawVCWTIy3brGPi6UklfCpIMfIjf7iGdXKHzg","tag":"XFBoMYUZodetZdvTiFvSkQ"}`,
 		},
 		{
-			Name: "JSON (Pretty)",
-			Func: func(m *jwe.Message) ([]byte, error) { return jwe.JSON(m, jwe.WithPrettyFormat(true)) },
+			Name:    "JSON (Pretty)",
+			Options: []jwe.EncryptOption{jwe.WithJSON(jwe.WithPretty(true))},
 			Expected: `{
   "ciphertext": "5eym8TW_c8SuK0ltJ3rpYIzOeDQz7TALvtu6UG9oMo4vpzs9tX_EFShS8iB7j6jiSdiwkIr3ajwQzaBtQD_A",
   "iv": "48V1_ALb6US04U3b",
@@ -224,32 +218,43 @@ func TestParse_RSAES_OAEP_AES_GCM(t *testing.T) {
 		},
 	}
 
-	for _, serializer := range serializers {
-		serializer := serializer
-		for _, compression := range []jwa.CompressionAlgorithm{jwa.NoCompress, jwa.Deflate} {
-			compression := compression
+	ntmpl := len(templates)
+	testcases := make([]struct {
+		Name     string
+		Options  []jwe.EncryptOption
+		Expected string
+	}, ntmpl*2)
+
+	for i, tmpl := range templates {
+		options := make([]jwe.EncryptOption, len(tmpl.Options))
+		copy(options, tmpl.Options)
+
+		for j, compression := range []jwa.CompressionAlgorithm{jwa.NoCompress, jwa.Deflate} {
 			compName := compression.String()
 			if compName == "" {
 				compName = "none"
 			}
-			t.Run(serializer.Name+" (compression="+compName+")", func(t *testing.T) {
-				jsonbuf, err := serializer.Func(msg)
-				if !assert.NoError(t, err, "serialize succeeded") {
-					return
-				}
+			tc := testcases[i+j]
+			tc.Name = tmpl.Name + " (compression=" + compName + ")"
+			tc.Expected = tmpl.Expected
+			tc.Options = append(options, jwe.WithCompress(compression))
+		}
+	}
 
-				if !assert.Equal(t, serializer.Expected, string(jsonbuf), "serialize result matches") {
-					jsonbuf, _ = jwe.JSON(msg, jwe.WithPrettyFormat(true))
-					t.Logf("%s", jsonbuf)
-					return
-				}
+	for _, tc := range testcases {
+		tc := tc
+		t.Run(tc.Name, func(t *testing.T) {
+			options := tc.Options
+			options = append(options, jwe.WithKey(jwa.RSA_OAEP, rawkey.PublicKey))
 
-				encrypted, err := jwe.Encrypt(plaintext, jwa.RSA_OAEP, rawkey.PublicKey, jwa.A256GCM, compression)
-				if !assert.NoError(t, err, "jwe.Encrypt should succeed") {
-					return
-				}
+			encrypted, err := jwe.Encrypt(plaintext, options...)
+			if !assert.NoError(t, err, "jwe.Encrypt should succeed") {
+				return
+			}
+			t.Logf("%s", encrypted)
 
-				plaintext, err = jwe.Decrypt(encrypted, jwa.RSA_OAEP, rawkey)
+			t.Run("WithKey", func(t *testing.T) {
+				plaintext, err = jwe.Decrypt(encrypted, jwe.WithKey(jwa.RSA_OAEP, rawkey))
 				if !assert.NoError(t, err, "jwe.Decrypt should succeed") {
 					return
 				}
@@ -258,7 +263,31 @@ func TestParse_RSAES_OAEP_AES_GCM(t *testing.T) {
 					return
 				}
 			})
-		}
+			t.Run("WithKeySet", func(t *testing.T) {
+				pkJwk, err := jwk.FromRaw(rawkey)
+				if !assert.NoError(t, err, `jwk.New should succeed`) {
+					return
+				}
+				// Keys are not going to be selected without an algorithm
+				_ = pkJwk.Set(jwe.AlgorithmKey, jwa.RSA_OAEP)
+				set := jwk.NewSet()
+				set.AddKey(pkJwk)
+
+				var used interface{}
+				plaintext, err = jwe.Decrypt(encrypted, jwe.WithKeySet(set, jwe.WithRequireKid(false)), jwe.WithKeyUsed(&used))
+				if !assert.NoError(t, err, "jwe.Decrypt should succeed") {
+					return
+				}
+
+				if !assert.Equal(t, payload, string(plaintext), "jwe.Decrypt should produce the same plaintext") {
+					return
+				}
+
+				if !assert.Equal(t, pkJwk, used) {
+					return
+				}
+			})
+		})
 	}
 
 	// Test direct marshaling and unmarshaling
@@ -290,18 +319,18 @@ func TestRoundtrip_RSAES_OAEP_AES_GCM(t *testing.T) {
 		110, 97, 116, 105, 111, 110, 46,
 	}
 
-	max := 100
+	iterations := 100
 	if testing.Short() {
-		max = 1
+		iterations = 1
 	}
 
-	for i := 0; i < max; i++ {
-		encrypted, err := jwe.Encrypt(plaintext, jwa.RSA_OAEP, &rsaPrivKey.PublicKey, jwa.A256GCM, jwa.NoCompress)
+	for i := 0; i < iterations; i++ {
+		encrypted, err := jwe.Encrypt(plaintext, jwe.WithKey(jwa.RSA_OAEP, &rsaPrivKey.PublicKey))
 		if !assert.NoError(t, err, "Encrypt should succeed") {
 			return
 		}
 
-		decrypted, err := jwe.Decrypt(encrypted, jwa.RSA_OAEP, rsaPrivKey)
+		decrypted, err := jwe.Decrypt(encrypted, jwe.WithKey(jwa.RSA_OAEP, rsaPrivKey))
 		if !assert.NoError(t, err, "Decrypt should succeed") {
 			return
 		}
@@ -318,18 +347,18 @@ func TestRoundtrip_RSA1_5_A128CBC_HS256(t *testing.T) {
 		112, 114, 111, 115, 112, 101, 114, 46,
 	}
 
-	max := 100
+	iterations := 100
 	if testing.Short() {
-		max = 1
+		iterations = 1
 	}
 
-	for i := 0; i < max; i++ {
-		encrypted, err := jwe.Encrypt(plaintext, jwa.RSA1_5, &rsaPrivKey.PublicKey, jwa.A128CBC_HS256, jwa.NoCompress)
+	for i := 0; i < iterations; i++ {
+		encrypted, err := jwe.Encrypt(plaintext, jwe.WithKey(jwa.RSA1_5, &rsaPrivKey.PublicKey), jwe.WithContentEncryption(jwa.A128CBC_HS256))
 		if !assert.NoError(t, err, "Encrypt is successful") {
 			return
 		}
 
-		decrypted, err := jwe.Decrypt(encrypted, jwa.RSA1_5, rsaPrivKey)
+		decrypted, err := jwe.Decrypt(encrypted, jwe.WithKey(jwa.RSA1_5, rsaPrivKey))
 		if !assert.NoError(t, err, "Decrypt successful") {
 			return
 		}
@@ -351,18 +380,18 @@ func TestEncode_A128KW_A128CBC_HS256(t *testing.T) {
 		25, 172, 32, 130, 225, 114, 26, 181, 138, 106, 254, 192, 95, 133, 74, 82,
 	}
 
-	max := 100
+	iterations := 100
 	if testing.Short() {
-		max = 1
+		iterations = 1
 	}
 
-	for i := 0; i < max; i++ {
-		encrypted, err := jwe.Encrypt(plaintext, jwa.A128KW, sharedkey, jwa.A128CBC_HS256, jwa.NoCompress)
+	for i := 0; i < iterations; i++ {
+		encrypted, err := jwe.Encrypt(plaintext, jwe.WithKey(jwa.A128KW, sharedkey), jwe.WithContentEncryption(jwa.A128CBC_HS256))
 		if !assert.NoError(t, err, "Encrypt is successful") {
 			return
 		}
 
-		decrypted, err := jwe.Decrypt(encrypted, jwa.A128KW, sharedkey)
+		decrypted, err := jwe.Decrypt(encrypted, jwe.WithKey(jwa.A128KW, sharedkey))
 		if !assert.NoError(t, err, "Decrypt successful") {
 			return
 		}
@@ -387,28 +416,17 @@ func testEncodeECDHWithKey(t *testing.T, privkey interface{}, pubkey interface{}
 	for _, alg := range algorithms {
 		alg := alg
 		t.Run(alg.String(), func(t *testing.T) {
-			encrypted, err := jwe.Encrypt(plaintext, alg, pubkey, jwa.A256GCM, jwa.NoCompress)
+			encrypted, err := jwe.Encrypt(plaintext, jwe.WithKey(alg, pubkey))
 			if !assert.NoError(t, err, "Encrypt succeeds") {
 				return
 			}
 
-			t.Logf("encrypted = %s", encrypted)
-
-			msg, err := jwe.Parse(encrypted)
+			_, err = jwe.Parse(encrypted)
 			if !assert.NoError(t, err, `jwe.Parse should succeed`) {
 				return
 			}
 
-			{
-				buf, _ := json.MarshalIndent(msg, "", "  ")
-				t.Logf("%s", buf)
-			}
-			{
-				buf, _ := json.MarshalIndent(msg.ProtectedHeaders(), "", "  ")
-				t.Logf("%s", buf)
-			}
-
-			decrypted, err := jwe.Decrypt(encrypted, alg, privkey)
+			decrypted, err := jwe.Decrypt(encrypted, jwe.WithKey(alg, privkey))
 			if !assert.NoError(t, err, "Decrypt succeeds") {
 				return
 			}
@@ -490,17 +508,10 @@ func Test_GHIssue207(t *testing.T) {
 				return
 			}
 
-			msg, err := jwe.ParseString(tc.Data)
-			if !assert.NoError(t, err, `jwe.ParseString should succeed`) {
-				return
-			}
-
-			{
-				buf, _ := json.MarshalIndent(msg, "", "  ")
-				t.Logf("%s", buf)
-			}
-
-			decrypted, err := msg.Decrypt(((msg.Recipients())[0]).Headers().Algorithm(), &key)
+			decrypted, err := jwe.Decrypt([]byte(tc.Data), jwe.WithKeyProvider(jwe.KeyProviderFunc(func(_ context.Context, sink jwe.KeySink, r jwe.Recipient, _ *jwe.Message) error {
+				sink.Key(r.Headers().Algorithm(), &key)
+				return nil
+			})))
 			if !assert.NoError(t, err, `jwe.Decrypt should succeed`) {
 				return
 			}
@@ -531,17 +542,21 @@ func TestEncode_Direct(t *testing.T) {
 		tc := tc
 		t.Run(tc.Algorithm.String(), func(t *testing.T) {
 			key := make([]byte, tc.KeySize)
-			_, err := rand.Read(key)
-			if !assert.NoError(t, err, "Key generation succeeds") {
-				return
+			/*
+				_, err := rand.Read(key)
+				if !assert.NoError(t, err, "Key generation succeeds") {
+					return
+				}*/
+			for n := 0; n < len(key); {
+				w := copy(key[n:], []byte(`12345678`))
+				n += w
 			}
 
-			encrypted, err := jwe.Encrypt(plaintext, jwa.DIRECT, key, tc.Algorithm, jwa.NoCompress)
-			if !assert.NoError(t, err, "Encrypt succeeds") {
+			encrypted, err := jwe.Encrypt(plaintext, jwe.WithKey(jwa.DIRECT, key), jwe.WithContentEncryption(tc.Algorithm))
+			if !assert.NoError(t, err, `jwe.Encrypt should succeed`) {
 				return
 			}
-
-			decrypted, err := jwe.Decrypt(encrypted, jwa.DIRECT, key)
+			decrypted, err := jwe.Decrypt(encrypted, jwe.WithKey(jwa.DIRECT, key))
 			if !assert.NoError(t, err, `jwe.Decrypt should succeed`) {
 				return
 			}
@@ -620,17 +635,7 @@ func TestDecodePredefined_Direct(t *testing.T) {
 				return
 			}
 
-			msg, err := jwe.ParseString(tc.Data)
-			if !assert.NoError(t, err, `jwe.ParseString should succeed`) {
-				return
-			}
-
-			{
-				buf, _ := json.MarshalIndent(msg, "", "  ")
-				t.Logf("%s", buf)
-			}
-
-			decrypted, err := msg.Decrypt(jwa.DIRECT, key)
+			decrypted, err := jwe.Decrypt([]byte(tc.Data), jwe.WithKey(jwa.DIRECT, key))
 			if !assert.NoError(t, err, `jwe.Decrypt should succeed`) {
 				return
 			}
@@ -671,9 +676,8 @@ func TestGHIssue230(t *testing.T) {
 
 func TestReadFile(t *testing.T) {
 	const s = `eyJhbGciOiJSU0EtT0FFUCIsImVuYyI6IkEyNTZHQ00ifQ.OKOawDo13gRp2ojaHV7LFpZcgV7T6DVZKTyKOMTYUmKoTCVJRgckCL9kiMT03JGeipsEdY3mx_etLbbWSrFr05kLzcSr4qKAq7YN7e9jwQRb23nfa6c9d-StnImGyFDbSv04uVuxIp5Zms1gNxKKK2Da14B8S4rzVRltdYwam_lDp5XnZAYpQdb76FdIKLaVmqgfwX7XWRxv2322i-vDxRfqNzo_tETKzpVLzfiwQyeyPGLBIO56YJ7eObdv0je81860ppamavo35UgoRdbYaBcoh9QcfylQr66oc6vFWXRcZ_ZT2LawVCWTIy3brGPi6UklfCpIMfIjf7iGdXKHzg.48V1_ALb6US04U3b.5eym8TW_c8SuK0ltJ3rpYIzOeDQz7TALvtu6UG9oMo4vpzs9tX_EFShS8iB7j6jiSdiwkIr3ajwQzaBtQD_A.XFBoMYUZodetZdvTiFvSkQ`
-	t.Parallel()
 
-	f, err := os.CreateTemp("", "test-read-file-*.jwe")
+	f, err := os.CreateTemp(t.TempDir(), "test-read-file-*.jwe")
 	if !assert.NoError(t, err, `os.CreateTemp should succeed`) {
 		return
 	}
@@ -707,7 +711,7 @@ func TestCustomField(t *testing.T) {
 	protected := jwe.NewHeaders()
 	protected.Set(`x-birthday`, string(bdaybytes))
 
-	encrypted, err := jwe.Encrypt(plaintext, jwa.RSA_OAEP, pubkey, jwa.A256GCM, jwa.NoCompress, jwe.WithProtectedHeaders(protected))
+	encrypted, err := jwe.Encrypt(plaintext, jwe.WithKey(jwa.RSA_OAEP, pubkey), jwe.WithProtectedHeaders(protected))
 	if !assert.NoError(t, err, `jwe.Encrypt should succeed`) {
 		return
 	}
@@ -768,7 +772,7 @@ func TestGH554(t *testing.T) {
 		return
 	}
 
-	encrypted, err := jwe.Encrypt([]byte(plaintext), jwa.ECDH_ES, pubkey, jwa.A256GCM, jwa.NoCompress)
+	encrypted, err := jwe.Encrypt([]byte(plaintext), jwe.WithKey(jwa.ECDH_ES, pubkey))
 	if !assert.NoError(t, err, `jwk.Encrypt() should succeed`) {
 		return
 	}
@@ -778,11 +782,6 @@ func TestGH554(t *testing.T) {
 		return
 	}
 
-	{
-		buf, _ := json.MarshalIndent(msg, "", "  ")
-		t.Logf("%s", buf)
-	}
-
 	recipients := msg.Recipients()
 
 	// The epk must have the same key ID as the original
@@ -792,6 +791,188 @@ func TestGH554(t *testing.T) {
 	}
 }
 
+func TestGH803(t *testing.T) {
+	privateKey, err := ecdsa.GenerateKey(elliptic.P384(), rand.Reader)
+	require.NoError(t, err, `ecdsa.GenerateKey should succeed`)
+
+	payload := []byte("Lorem Ipsum")
+	apu := []byte(`Alice`)
+	apv := []byte(`Bob`)
+	hdrs := jwe.NewHeaders()
+	hdrs.Set(jwe.AgreementPartyUInfoKey, apu)
+	hdrs.Set(jwe.AgreementPartyVInfoKey, apv)
+	encrypted, err := jwe.Encrypt(
+		payload,
+		jwe.WithJSON(),
+		jwe.WithKey(jwa.ECDH_ES, privateKey.PublicKey, jwe.WithPerRecipientHeaders(hdrs)),
+		jwe.WithContentEncryption(jwa.A128GCM),
+	)
+	require.NoError(t, err, `jwe.Encrypt should succeed`)
+
+	var msg jwe.Message
+	decrypted, err := jwe.Decrypt(
+		encrypted,
+		jwe.WithKey(jwa.ECDH_ES, privateKey),
+		jwe.WithMessage(&msg),
+	)
+	require.NoError(t, err, `jwe.Decrypt should succeed`)
+	require.Equal(t, payload, decrypted, `decrypt messages match`)
+	require.Equal(t, apu, msg.ProtectedHeaders().AgreementPartyUInfo())
+	require.Equal(t, apv, msg.ProtectedHeaders().AgreementPartyVInfo())
+}
+
+func TestGH840(t *testing.T) {
+	// Go 1.19+ panics if elliptic curve operations are called against
+	// a point that's _NOT_ on the curve
+	untrustedJWK := []byte(`{
+		"kty": "EC",
+		"crv": "P-256",
+		"x": "MKBCTNIcKUSDii11ySs3526iDZ8AiTo7Tu6KPAqx7D4",
+		"y": "4Etl6SRW2YiLUrN5vfvVHuhp7x8PxltmWWlbbM4IFyM",
+		"d": "870MB6gfuTJ4HtUnUvYMyJpr5eUZNP4Bk43bVdj3eAE"
+	}`)
+
+	privkey, err := jwk.ParseKey(untrustedJWK)
+	require.NoError(t, err, `jwk.ParseKey should succeed`)
+
+	pubkey, err := privkey.PublicKey()
+	require.NoError(t, err, `privkey.PublicKey should succeed`)
+
+	const payload = `Lorem ipsum`
+	_, err = jwe.Encrypt([]byte(payload), jwe.WithKey(jwa.ECDH_ES_A128KW, pubkey))
+	require.Error(t, err, `jwe.Encrypt should fail (instead of panic)`)
+}
+
+type dummyKeyEncrypterDecrypter struct {
+	key []byte
+}
+
+func (kd *dummyKeyEncrypterDecrypter) DecryptKey(_ jwa.KeyEncryptionAlgorithm, cek []byte, _ jwe.Recipient, _ *jwe.Message) ([]byte, error) {
+	return bytes.TrimSuffix(cek, kd.key), nil
+}
+
+func (kd *dummyKeyEncrypterDecrypter) Algorithm() jwa.KeyEncryptionAlgorithm {
+	return jwa.A128GCMKW
+}
+
+func (kd *dummyKeyEncrypterDecrypter) EncryptKey(key []byte) ([]byte, error) {
+	return append(key, kd.key...), nil
+}
+
+var _ jwe.KeyEncrypter = (*dummyKeyEncrypterDecrypter)(nil)
+
+func TestGH924(t *testing.T) {
+	sharedKey := []byte("abra-kadabra")
+
+	ked := &dummyKeyEncrypterDecrypter{key: sharedKey}
+
+	payload := []byte("Lorem Ipsum")
+	encrypted, err := jwe.Encrypt(
+		payload,
+		jwe.WithJSON(),
+		jwe.WithKey(jwa.A128GCMKW, ked),
+		jwe.WithContentEncryption(jwa.A128GCM),
+	)
+	require.NoError(t, err, `jwe.Encrypt should succeed`)
+
+	var msg jwe.Message
+	decrypted, err := jwe.Decrypt(
+		encrypted,
+		jwe.WithKey(jwa.A128GCMKW, ked),
+		jwe.WithMessage(&msg),
+	)
+	require.NoError(t, err, `jwe.Decrypt should succeed`)
+	require.Equal(t, payload, decrypted, `decrypt messages match`)
+}
+
+func TestGH1001(t *testing.T) {
+	rawKey, err := jwxtest.GenerateRsaKey()
+	require.NoError(t, err, `jwxtest.GenerateRsaKey should succeed`)
+
+	encrypted, err := jwe.Encrypt([]byte("Lorem Ipsum"), jwe.WithKey(jwa.RSA_OAEP, rawKey.PublicKey))
+	require.NoError(t, err, `jwe.Encrypt should succeed`)
+	var cek []byte
+	decrypted, err := jwe.Decrypt(encrypted, jwe.WithKey(jwa.RSA_OAEP, rawKey), jwe.WithCEK(&cek))
+	require.NoError(t, err, `jwe.Decrypt should succeed`)
+
+	require.Equal(t, "Lorem Ipsum", string(decrypted), `decrypted message should match`)
+	require.NotNil(t, cek, `cek should not be nil`)
+
+	reEncrypted, err := jwe.EncryptStatic([]byte("Lorem Ipsum"), cek, jwe.WithKey(jwa.RSA_OAEP, rawKey.PublicKey))
+	require.NoError(t, err, `jwe.EncryptStatic should succeed`)
+
+	// sanity. empty CEKs should be rejected
+	_, err = jwe.EncryptStatic([]byte("Lorem Ipsum"), nil, jwe.WithKey(jwa.RSA_OAEP, rawKey.PublicKey))
+	require.Error(t, err, `jwe.Encryptstatic should fail with empty cek`)
+
+	cek = []byte(nil)
+	decrypted, err = jwe.Decrypt(reEncrypted, jwe.WithKey(jwa.RSA_OAEP, rawKey), jwe.WithCEK(&cek))
+	require.NoError(t, err, `jwe.Decrypt should succeed`)
+
+	require.Equal(t, "Lorem Ipsum", string(decrypted), `decrypted message should match`)
+	require.NotNil(t, cek, `cek should not be nil`)
+}
+
+func TestGHSA_7f9x_gw85_8grf(t *testing.T) {
+	token := []byte("eyJhbGciOiJQQkVTMi1IUzI1NitBMTI4S1ciLCJlbmMiOiJBMjU2R0NNIiwicDJjIjoyMDAwMDAwMDAwLCJwMnMiOiJNNzczSnlmV2xlX2FsSXNrc0NOTU9BIn0=.S8B1kXdIR7BM6i_TaGsgqEOxU-1Sgdakp4mHq7UVhn-_REzOiGz2gg.gU_LfzhBXtQdwYjh.9QUIS-RWkLc.m9TudmzUoCzDhHsGGfzmCA")
+	key, err := jwk.FromRaw([]byte(`abcdefg`))
+	require.NoError(t, err, `jwk.FromRaw should succeed`)
+
+	{
+		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+		defer cancel()
+
+		done := make(chan struct{})
+		go func(t *testing.T, done chan struct{}) {
+			_, err := jwe.Decrypt(token, jwe.WithKey(jwa.PBES2_HS256_A128KW, key))
+			require.Error(t, err, `jwe.Decrypt should fail`)
+			close(done)
+		}(t, done)
+
+		select {
+		case <-done:
+		case <-ctx.Done():
+			require.Fail(t, "jwe.Decrypt should not block")
+		}
+	}
+
+	// NOTE: HAS GLOBAL EFFECT
+	// Should allow for timeout to occur
+	jwe.Settings(jwe.WithMaxPBES2Count(math.MaxInt32))
+
+	// put it back to normal after the test
+	defer jwe.Settings(jwe.WithMaxPBES2Count(10000))
+	{
+		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+		defer cancel()
+
+		done := make(chan struct{})
+		go func(done chan struct{}) {
+			_, _ = jwe.Decrypt(token, jwe.WithKey(jwa.PBES2_HS256_A128KW, key))
+			close(done)
+		}(done)
+
+		select {
+		case <-done:
+			require.Fail(t, "jwe.Decrypt should block")
+		case <-ctx.Done():
+			// timeout occurred as it should
+		}
+	}
+}
+
+func TestMaxBufferSize(t *testing.T) {
+	// NOTE: This has GLOBAL EFFECT
+	jwe.Settings(jwe.WithMaxBufferSize(1))
+	defer jwe.Settings(jwe.WithMaxBufferSize(0))
+
+	key, err := jwxtest.GenerateRsaJwk()
+	require.NoError(t, err, `jwxtest.GenerateRsaJwk should succeed`)
+
+	_, err = jwe.Encrypt([]byte("Lorem Ipsum"), jwe.WithContentEncryption(jwa.A128CBC_HS256), jwe.WithKey(jwa.RSA_OAEP, key))
+	require.Error(t, err, `jwe.Encrypt should fail`)
+}
+
 func TestMaxDecompressBufferSize(t *testing.T) {
 	// This payload size is intentionally set to a small value to avoid
 	// causing problems for regular users and CI/CD systems. If you wish to
@@ -811,41 +992,64 @@ func TestMaxDecompressBufferSize(t *testing.T) {
 	payload := strings.Repeat("x", payloadSize)
 
 	testcases := []struct {
-		Name      string
-		MaxSize   int64
-		PublicKey *rsa.PublicKey
-		Error     bool
+		Name                  string
+		GlobalMaxSize         int64
+		PublicKey             *rsa.PublicKey
+		Error                 bool
+		ProcessDecryptOptions func([]jwe.DecryptOption) []jwe.DecryptOption
 	}{
 		// This should work, because we set the MaxSize to be large (==payload size)
 		{
-			Name:      "same as payload size",
-			MaxSize:   payloadSize,
-			PublicKey: pubkey,
+			Name:          "same as payload size",
+			GlobalMaxSize: payloadSize,
+			PublicKey:     pubkey,
 		},
 		// This should fail, because we set the GlobalMaxSize to be smaller than the payload size
 		{
-			Name:      "smaller than payload size",
-			MaxSize:   payloadSize - 1,
-			PublicKey: pubkey,
-			Error:     true,
+			Name:          "smaller than payload size",
+			GlobalMaxSize: payloadSize - 1,
+			PublicKey:     pubkey,
+			Error:         true,
 		},
 		// This should fail, because the public key does not match the
 		// private key used to decrypt the payload. In essence this way
 		// we do NOT trigger the root cause of this issue, but we bail out early
 		{
-			Name:      "Wrong PublicKey",
-			PublicKey: wrongPubkey,
-			MaxSize:   payloadSize,
-			Error:     true,
+			Name:          "Wrong PublicKey",
+			GlobalMaxSize: payloadSize,
+			PublicKey:     wrongPubkey,
+			Error:         true,
+		},
+		{
+			Name:          "global=payloadSize-1, per-call=payloadSize",
+			GlobalMaxSize: payloadSize - 1,
+			PublicKey:     pubkey,
+			ProcessDecryptOptions: func(options []jwe.DecryptOption) []jwe.DecryptOption {
+				return append(options, jwe.WithMaxDecompressBufferSize(payloadSize))
+			},
+		},
+		// This should be the last test case to put the value back to default :)
+		{
+			Name:          "Default 10MB globally",
+			GlobalMaxSize: 10 * 1024 * 1024,
+			PublicKey:     pubkey,
 		},
 	}
 	for _, tc := range testcases {
 		tc := tc
 		t.Run(tc.Name, func(t *testing.T) {
-			encrypted, err := jwe.Encrypt([]byte(payload), jwa.RSA_OAEP, tc.PublicKey, "A128CBC-HS256", jwa.Deflate)
+			jwe.Settings(jwe.WithMaxDecompressBufferSize(tc.GlobalMaxSize))
+
+			encrypted, err := jwe.Encrypt([]byte(payload), jwe.WithKey(jwa.RSA_OAEP, tc.PublicKey), jwe.WithContentEncryption("A128CBC-HS256"), jwe.WithCompress(jwa.Deflate))
+
 			require.NoError(t, err, `jwe.Encrypt should succeed`)
 
-			_, err = jwe.Decrypt(encrypted, jwa.RSA_OAEP, privkey, jwe.WithMaxDecompressBufferSize(tc.MaxSize))
+			decryptOptions := []jwe.DecryptOption{jwe.WithKey(jwa.RSA_OAEP, privkey)}
+
+			if fn := tc.ProcessDecryptOptions; fn != nil {
+				decryptOptions = fn(decryptOptions)
+			}
+			_, err = jwe.Decrypt(encrypted, decryptOptions...)
 			if tc.Error {
 				require.Error(t, err, `jwe.Decrypt should fail`)
 			} else {
