@@ -8,9 +8,10 @@
 package openid
 
 import (
-	"github.com/lestrrat-go/jwx/internal/json"
-	"github.com/lestrrat-go/jwx/jwt"
-	"github.com/pkg/errors"
+	"fmt"
+
+	"github.com/lestrrat-go/jwx/v2/internal/json"
+	"github.com/lestrrat-go/jwx/v2/jwt"
 )
 
 var registry = json.NewRegistry()
@@ -22,7 +23,7 @@ func (t *stdToken) Clone() (jwt.Token, error) {
 		//nolint:forcetypeassert
 		key := pair.Key.(string)
 		if err := dst.Set(key, pair.Value); err != nil {
-			return nil, errors.Wrapf(err, `failed to set %s`, key)
+			return nil, fmt.Errorf(`failed to set %s: %w`, key, err)
 		}
 	}
 	return dst, nil
