@@ -7,14 +7,15 @@ import (
 	"testing"
 	"time"
 
-	"github.com/lestrrat-go/jwx/internal/json"
-	"github.com/lestrrat-go/jwx/internal/jwxtest"
+	"github.com/lestrrat-go/jwx/v2/internal/json"
+	"github.com/lestrrat-go/jwx/v2/internal/jwxtest"
 
-	"github.com/lestrrat-go/jwx/jwa"
-	"github.com/lestrrat-go/jwx/jwt"
-	"github.com/lestrrat-go/jwx/jwt/internal/types"
-	"github.com/lestrrat-go/jwx/jwt/openid"
+	"github.com/lestrrat-go/jwx/v2/jwa"
+	"github.com/lestrrat-go/jwx/v2/jwt"
+	"github.com/lestrrat-go/jwx/v2/jwt/internal/types"
+	"github.com/lestrrat-go/jwx/v2/jwt/openid"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 const aLongLongTimeAgo = 233431200
@@ -247,10 +248,10 @@ func TestOpenIDClaims(t *testing.T) {
 			},
 		},
 		{
-			Value: "https://github.com/lestrrat-go/jwx",
+			Value: "https://github.com/lestrrat-go/jwx/v2",
 			Key:   openid.ProfileKey,
 			Check: func(token openid.Token) {
-				assert.Equal(t, token.Profile(), "https://github.com/lestrrat-go/jwx")
+				assert.Equal(t, token.Profile(), "https://github.com/lestrrat-go/jwx/v2")
 			},
 		},
 		{
@@ -261,10 +262,10 @@ func TestOpenIDClaims(t *testing.T) {
 			},
 		},
 		{
-			Value: "https://github.com/lestrrat-go/jwx",
+			Value: "https://github.com/lestrrat-go/jwx/v2",
 			Key:   openid.WebsiteKey,
 			Check: func(token openid.Token) {
-				assert.Equal(t, token.Website(), "https://github.com/lestrrat-go/jwx")
+				assert.Equal(t, token.Website(), "https://github.com/lestrrat-go/jwx/v2")
 			},
 		},
 		{
@@ -406,11 +407,13 @@ func TestOpenIDClaims(t *testing.T) {
 	var tokens []openidTokTestCase
 
 	{ // one with Set()
-		token := openid.New()
+		b := openid.NewBuilder()
 		for name, value := range data {
-			if !assert.NoError(t, token.Set(name, value), `token.Set should succeed`) {
-				return
-			}
+			b.Claim(name, value)
+		}
+		token, err := b.Build()
+		if !assert.NoError(t, err, `b.Build() should succeed`) {
+			return
 		}
 		tokens = append(tokens, openidTokTestCase{Name: `token constructed by calling Set()`, Token: token})
 	}
@@ -450,12 +453,12 @@ func TestOpenIDClaims(t *testing.T) {
 			if !assert.NoError(t, err, `rsa.GeneraKey should succeed`) {
 				return
 			}
-			signed, err := jwt.Sign(token, alg, key)
+			signed, err := jwt.Sign(token, jwt.WithKey(alg, key))
 			if !assert.NoError(t, err, `jwt.Sign should succeed`) {
 				return
 			}
 
-			tokenTmp, err := jwt.Parse(signed, jwt.WithToken(openid.New()), jwt.WithVerify(alg, &key.PublicKey))
+			tokenTmp, err := jwt.Parse(signed, jwt.WithToken(openid.New()), jwt.WithKey(alg, &key.PublicKey), jwt.WithValidate(false))
 			if !assert.NoError(t, err, `parsing the token via jwt.Parse should succeed`) {
 				return
 			}
@@ -674,3 +677,32 @@ func TestKeys(t *testing.T) {
 	at.Equal(`website`, openid.WebsiteKey)
 	at.Equal(`zoneinfo`, openid.ZoneinfoKey)
 }
+
+func TestGH734(t *testing.T) {
+	const src = `{
+    "nickname": "miniscruff",
+    "updated_at": "2022-05-06T04:57:24.367Z",
+    "email_verified": true
+  }`
+
+	expected, _ := time.Parse(time.RFC3339, "2022-05-06T04:57:24.367Z")
+	for _, pedantic := range []bool{true, false} {
+		t.Run(fmt.Sprintf("pedantic=%t", pedantic), func(t *testing.T) {
+			jwt.Settings(jwt.WithNumericDateParsePedantic(pedantic))
+			tok := openid.New()
+			_, err := jwt.Parse(
+				[]byte(src),
+				jwt.WithToken(tok),
+				jwt.WithVerify(false),
+				jwt.WithValidate(false),
+			)
+			if pedantic {
+				require.Error(t, err, `jwt.Parse should fail for pedantic parser`)
+			} else {
+				require.NoError(t, err, `jwt.Parse should succeed`)
+				require.Equal(t, expected, tok.UpdatedAt(), `updated_at should match`)
+			}
+		})
+	}
+	jwt.Settings(jwt.WithNumericDateParsePedantic(false))
+}
